require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xmlconfigfile',
  dscmeta_resource_friendly_name: 'XMLConfigFile',
  dscmeta_resource_name: 'xXMLConfigFile',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xXMLConfigFile',
  dscmeta_module_version: '2.0.0.2',
  docs: 'The DSC XMLConfigFile resource type.
         Automatically generated from version 2.0.0.2',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_attribute1: {
      type: 'Optional[String]',
      desc: 'Required for key/value pair. Default: key',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_configpath: {
      type: 'String',
      desc: 'Path to config file',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_enforcenullxmlns: {
      type: 'Optional[Boolean]',
      desc: 'Enforce no XMLNS',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_iselementtextvalue: {
      type: 'Optional[Boolean]',
      desc: 'Name is element only',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_nsprefix: {
      type: 'Optional[String]',
      desc: 'The prefix of the XmlNamespaceManager, which is used in the XPath',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_value: {
      type: 'Optional[String]',
      desc: 'Value of the attribute',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_xmlns: {
      type: 'Optional[String]',
      desc: 'Define XmlNamespaceManager. If omitted the resource determines one from file.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Ensure Present or Absent',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'Optional[String]',
      desc: 'Name of the attribute/element',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_dobackup: {
      type: 'Optional[Boolean]',
      desc: 'Whether to create a backup before changing a file',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_xpath: {
      type: 'String',
      desc: 'XPath to use',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_attribute2: {
      type: 'Optional[String]',
      desc: 'Required for key/value pair. Default: value',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_isattribute: {
      type: 'Optional[Boolean]',
      desc: 'Name is attribute',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
  },
)
